/*
 * n01
 *
 * n01_ghost.c
 *
 * Copyright (C) 1996-2005 by Nakashima Tomoaki. All rights reserved.
 *		http://www.nakka.com/
 *		nakka@nakka.com
 */

/* Include Files */
#define _INC_OLE
#include <windows.h>
#undef  _INC_OLE
#include <tchar.h>
#include <shlobj.h>

#include "../n01_plugin.h"
#include "Memory.h"
#include "Message.h"

#include "resource.h"

/* Define */
#define	INI_FILE_NAME				TEXT("n01_ghost.ini")

#define WM_PLAYER_REDRAW			(WM_APP + 300)
#define WM_PLAYER_DRAW_INIT			(WM_APP + 301)

#define GHOST_MAX_ROUND				100

/* Global Variables */
HINSTANCE hInst;

static TCHAR ini_path[BUF_SIZE];
static TCHAR csv_path[MAX_PATH];
static int g_start_score;
static TCHAR g_name[BUF_SIZE];
static int g_random;
static int g_hidden;
static int g_first;

typedef struct _GHOST_SCORE_INFO {
	TCHAR name[NAME_SIZE];
	int start_score;
	TYPE_SCORE score[GHOST_MAX_ROUND];
} GHOST_SCORE_INFO;

typedef struct _GHOST_INFO {
	BOOL ghost;
	int index;

	TCHAR name[NAME_SIZE];
	int start_score;
	int random;
	int hidden;
} GHOST_INFO;

typedef struct _GHOST {
	int player;

	GHOST_INFO ghi[2];
	GHOST_SCORE_INFO *ghsi;
	int ghsi_count;
} GHOST;

/* Local Function Prototypes */
static BOOL get_csv(const TCHAR *path, GHOST *ghost);
static BOOL get_csv_file(const TCHAR *file_path, GHOST *ghost);
static BOOL folder_select(const HWND hWnd, const TCHAR *title, TCHAR *ret);
static void update_start_score(const HWND hDlg, const GHOST *ghost);
static void update_player_name(const HWND hDlg, const GHOST *ghost, const int start_score);
static BOOL CALLBACK set_ghost_proc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam);

/*
 * DllMain - C
 */
int WINAPI DllMain(HINSTANCE hInstance, DWORD fdwReason, PVOID pvReserved)
{
	TCHAR work_path[MAX_PATH];
	TCHAR *p, *r;

	switch (fdwReason) {
	case DLL_PROCESS_ATTACH:
		hInst = hInstance;
		// AvP[ṼpX擾
		GetModuleFileName(hInst, work_path, MAX_PATH - 1);
		for (p = r = work_path; *p != TEXT('\0'); p++) {
#ifndef UNICODE
			if (IsDBCSLeadByte((BYTE)*p) == TRUE) {
				p++;
				continue;
			}
#endif	// UNICODE
			if (*p == TEXT('\\') || *p == TEXT('/')) {
				r = p;
			}
		}
		*r = TEXT('\0');
		wsprintf(ini_path, TEXT("%s\\%s"), work_path, INI_FILE_NAME);

		GetPrivateProfileString(TEXT("CSV"), TEXT("path"), TEXT(""), csv_path, BUF_SIZE - 1, ini_path);
		g_start_score = GetPrivateProfileInt(TEXT("OPTION"), TEXT("start_score"), 501, ini_path);
		GetPrivateProfileString(TEXT("OPTION"), TEXT("name"), TEXT(""), g_name, BUF_SIZE - 1, ini_path);
		g_random = GetPrivateProfileInt(TEXT("OPTION"), TEXT("random"), 0, ini_path);
		g_hidden = GetPrivateProfileInt(TEXT("OPTION"), TEXT("hidden"), 0, ini_path);
		g_first = GetPrivateProfileInt(TEXT("OPTION"), TEXT("first"), 1, ini_path);
		break;

	case DLL_PROCESS_DETACH:
		break;

	case DLL_THREAD_ATTACH:
	case DLL_THREAD_DETACH:
		break;
	}
	return TRUE;
}

/*
 * get_plugin_info - vOC擾
 *
 *	:
 *		hWnd - ĂяoEBhE
 *		index - 擾̃CfbNX (0`)
 *		pgi - vOC擾
 *
 *	߂l:
 *		TRUE - Ɏ擾vOC
 *		FALSE - 擾̏I
 */
__declspec(dllexport) BOOL CALLBACK get_plugin_info(const HWND hWnd, const int index, PLUGIN_GET_INFO *pgi)
{
	switch (index) {
	case 0:
		lstrcpy(pgi->title, TEXT("Player &Ghost"));
		lstrcpy(pgi->func_name, TEXT("n01_ghost"));
		lstrcpy(pgi->cmd_line, TEXT(""));
		return TRUE;

	case 1:
		return FALSE;
	}
	return FALSE;
}

/*
 * n01_ghost_initialize - vOC̏
 *
 *	:
 *		hWnd - ĂяoEBhE
 *		pei - vOCs
 *
 *	߂l:
 *		vOČĂяo@
 *
 *	l:
 *		֐ vOC_initialize
 */
__declspec(dllexport) BOOL CALLBACK n01_ghost_initialize(const HWND hWnd, PLUGIN_EXEC_INFO *pei)
{
	// ߂lɌĂяo@̑gݍ킹ݒ
	return (CALLTYPE_MENU | CALLTYPE_SET_START | CALLTYPE_LEG_START | CALLTYPE_INPUT_START | CALLTYPE_END | CALLTYPE_PREV_LEG);

}

/*
 * n01_ghost - vOC
 *
 *	:
 *		hWnd - ĂяoEBhE
 *		pei - vOCs
 *		param - Ăяo
 *
 *	߂l:
 *		PLUGIN_
 */
__declspec(dllexport) int CALLBACK n01_ghost(const HWND hWnd, const int call_type, PLUGIN_EXEC_INFO *pei, int param)
{
	OPTION_INFO *op;
	WINDOW_INFO *wi;
	GAME_INFO *gi;
	GHOST *ghost;
	int x, y;
	int i, j;

	switch (call_type) {
	case CALLTYPE_MENU:
		// S[Xgݒ̊m
		ghost = (GHOST *)pei->lParam1;
		if (ghost == NULL) {
			ghost = (GHOST *)mem_calloc(sizeof(GHOST));
			if (ghost == NULL) {
				break;
			}
			pei->lParam1 = (LPARAM)ghost;
		}
		// ݒ
		if (DialogBoxParam(hInst, MAKEINTRESOURCE(IDD_DIALOG_GHOST), hWnd, set_ghost_proc, (LPARAM)ghost) == FALSE) {
			break;
		}
		// ݒ擾
		ghost->ghi[g_first].start_score = g_start_score;
		lstrcpy(ghost->ghi[g_first].name, g_name);
		ghost->ghi[g_first].random = g_random;
		ghost->ghi[g_first].hidden = g_hidden;
		ghost->ghi[g_first].ghost = TRUE;
		ghost->ghi[!g_first].ghost = FALSE;

		// vC[̏
		op = (OPTION_INFO *)SendMessage(hWnd, WM_N01_GET_OPTION, 0, 0);
		ghost->player = 0;
		for (i = 0; i < 2; i++) {
			if (ghost->ghi[i].ghost == TRUE) {
				pei->si->player[i].lock = TRUE;
				ghost->player |= (i + 1);
				op->gi.player_start_score[i] = ghost->ghi[i].start_score;
				if (ghost->ghi[i].random == 1 && ghost->ghi[i].hidden == 1) {
					lstrcpy(op->gi.player_name[i], TEXT("ghost"));
				} else {
					lstrcpy(op->gi.player_name[i], ghost->ghi[i].name);
				}
				op->gi.com[i] = FALSE;
			}
		}
		// JntO
		pei->lParam2 = 1;

		// Q[Jn
		SendMessage(hWnd, WM_N01_GAME_ON, FALSE, 0);
		break;

	case CALLTYPE_SET_START:
		ghost = (GHOST *)pei->lParam1;
		if (ghost == NULL) {
			break;
		}
		if (param == 0 && pei->lParam2 == 0) {
			// S[Xg̉
			ghost->player = 0;
		} else {
			// JntȌ
			pei->lParam2 = 0;

			if (ghost->ghsi_count > 0) {
				// ̏
				srand((unsigned)GetTickCount());
				wi = (WINDOW_INFO *)SendMessage(hWnd, WM_N01_GET_WINDOW, 0, 0);
				gi = (GAME_INFO *)SendMessage(hWnd, WM_N01_GET_GAME_OPTION, 0, 0);
				for (j = 0; j < 2; j++) {
					if (ghost->player & (j + 1)) {
						pei->si->player[j].lock = TRUE;
						if (ghost->ghi[j].random == 1 && gi->leg_limit == 1) {
							// vC[ύX
							for (i = 0; i < ghost->ghsi_count; i++) {
								if (i != ghost->ghi[j].index &&
									ghost->ghsi[i].start_score == ghost->ghi[j].start_score &&
									lstrcmpi(ghost->ghsi[i].name, ghost->ghi[j].name) != 0) {
									break;
								}
							}
							if (i < ghost->ghsi_count) {
								while (1) {
									i = rand() % ghost->ghsi_count;
									if (i != ghost->ghi[j].index &&
										ghost->ghsi[i].start_score == ghost->ghi[j].start_score &&
										lstrcmpi(ghost->ghsi[i].name, ghost->ghi[j].name) != 0) {
										lstrcpy(ghost->ghi[j].name, ghost->ghsi[i].name);
										break;
									}
								}
							}
							if (ghost->ghi[j].hidden == 0) {
								// vC[XV
								lstrcpy(pei->si->player[j].name, ghost->ghi[j].name);
								SendMessage(wi->score_player_wnd[j], WM_PLAYER_DRAW_INIT, 0, 0);
								SendMessage(wi->score_player_wnd[j], WM_PLAYER_REDRAW, 0, 0);
							}
						}
					}
				}
			}
		}
		break;

	case CALLTYPE_LEG_START:
		ghost = (GHOST *)pei->lParam1;
		if (ghost == NULL || ghost->ghsi == NULL) {
			break;
		}
		if (ghost->ghsi_count > 0) {
			// ̏
			srand((unsigned)GetTickCount());
			wi = (WINDOW_INFO *)SendMessage(hWnd, WM_N01_GET_WINDOW, 0, 0);
			gi = (GAME_INFO *)SendMessage(hWnd, WM_N01_GET_GAME_OPTION, 0, 0);
			for (j = 0; j < 2; j++) {
				if (ghost->player & (j + 1)) {
					pei->si->player[j].lock = TRUE;
					if (ghost->ghi[j].random == 0 || gi->leg_limit == 1) {
						// XRA_ɑI
						for (i = 0; i < ghost->ghsi_count; i++) {
							if (i != ghost->ghi[j].index &&
								ghost->ghsi[i].start_score == ghost->ghi[j].start_score &&
								lstrcmpi(ghost->ghsi[i].name, ghost->ghi[j].name) == 0) {
								break;
							}
						}
						if (i < ghost->ghsi_count) {
							while (1) {
								i = rand() % ghost->ghsi_count;
								if (i != ghost->ghi[j].index &&
									ghost->ghsi[i].start_score == ghost->ghi[j].start_score &&
									lstrcmpi(ghost->ghsi[i].name, ghost->ghi[j].name) == 0) {
									ghost->ghi[j].index = i;
									break;
								}
							}
						}
					} else {
						// vC[ύX
						for (i = 0; i < ghost->ghsi_count; i++) {
							if (i != ghost->ghi[j].index &&
								ghost->ghsi[i].start_score == ghost->ghi[j].start_score) {
								break;
							}
						}
						if (i < ghost->ghsi_count) {
							while (1) {
								i = rand() % ghost->ghsi_count;
								if (i != ghost->ghi[j].index && ghost->ghsi[i].start_score == ghost->ghi[j].start_score) {
									ghost->ghi[j].index = i;
									lstrcpy(ghost->ghi[j].name, ghost->ghsi[i].name);
									break;
								}
							}
						}
						if (ghost->ghi[j].hidden == 0) {
							// vC[XV
							lstrcpy(pei->si->player[j].name, ghost->ghi[j].name);
							SendMessage(wi->score_player_wnd[j], WM_PLAYER_DRAW_INIT, 0, 0);
							SendMessage(wi->score_player_wnd[j], WM_PLAYER_REDRAW, 0, 0);
						}
					}
				}
			}
		}
		break;

	case CALLTYPE_INPUT_START:
		ghost = (GHOST *)pei->lParam1;
		if (ghost == NULL || ghost->ghsi == NULL) {
			break;
		}
		SendMessage(hWnd, WM_N01_GET_INPUT_POS, (WPARAM)&x, (LPARAM)&y);
		if (pei->si->player[x].lock != TRUE ||
			(x == 0 && !(ghost->player & 1) ||
			x == 1 && !(ghost->player & 2))) {
			break;
		}
		if (pei->si->leg[pei->si->current_leg].current_round == y &&
			pei->si->leg[pei->si->current_leg].current_player == x) {
			pei->si->player[x].check_out_mode = FALSE;
			PostMessage(hWnd, WM_N01_SCORE_INPUT, ghost->ghsi[ghost->ghi[x].index].score[y], 0);
			return PLUGIN_CANCEL;
		}
		break;

	case CALLTYPE_END:
		ghost = (GHOST *)pei->lParam1;
		if (ghost != NULL) {
			mem_free(&ghost->ghsi);
			mem_free(&ghost);
		}
		pei->lParam1 = 0;
		break;

	case CALLTYPE_PREV_LEG:
		ghost = (GHOST *)pei->lParam1;
		if (ghost == NULL || ghost->ghsi == NULL || ghost->player == 0) {
			break;
		}
		return PLUGIN_CANCEL;
	}
	return PLUGIN_SUCCEED;
}

/*
 * get_csv - CSVēǂݍ
 */
static BOOL get_csv(const TCHAR *path, GHOST *ghost)
{
	HCURSOR old_cursor;
	WIN32_FIND_DATA find_data;
	HANDLE h_find_file;
	TCHAR find_path[MAX_PATH];
	TCHAR buf[MAX_PATH];

	old_cursor = SetCursor(LoadCursor(NULL, IDC_WAIT));
	mem_free(&ghost->ghsi);
	ghost->ghsi_count = 0;

	wsprintf(find_path, TEXT("%s\\*.csv"), path);
	if((h_find_file = FindFirstFile(find_path, &find_data)) == INVALID_HANDLE_VALUE){
		SetCursor(old_cursor);
		return FALSE;
	}
	do{
		if(find_data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY){
			continue;
		}
		wsprintf(buf, TEXT("%s\\%s"), path, find_data.cFileName);
		get_csv_file(buf, ghost);
	} while(FindNextFile(h_find_file, &find_data) == TRUE);
	FindClose(h_find_file);
	SetCursor(old_cursor);
	return TRUE;
}

/*
 * get_csv_file - CSVt@Cǂݍ
 */
static BOOL get_csv_file(const TCHAR *file_path, GHOST *ghost)
{
	HANDLE hFile;
	DWORD size_low, size_high;
	DWORD ret;
	TCHAR *buf;
	TCHAR *p, *r;
	char *cbuf;
	long file_size;
	int round;
	int winner;
	GHOST_SCORE_INFO tmp_ghsi[2];
	GHOST_SCORE_INFO *alloc_ghsi;
#ifdef UNICODE
	long len;
#endif

	// t@CJ
	hFile = CreateFile(file_path, GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE, 0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if (hFile == NULL || hFile == (HANDLE)-1) {
		return FALSE;
	}
	// mۂTCY̎擾
	if ((size_low = GetFileSize(hFile, &size_high)) == 0xFFFFFFFF) {
		CloseHandle(hFile);
		return FALSE;
	}
	file_size = (long)size_low;

	// ǂݎ̈̊m
	if ((cbuf = (char *)mem_alloc(file_size + 1)) == NULL) {
		CloseHandle(hFile);
		return FALSE;
	}
	// t@Cǂݍ
	if (ReadFile(hFile, cbuf, size_low, &ret, NULL) == FALSE) {
		mem_free(&cbuf);
		CloseHandle(hFile);
		return FALSE;
	}
	CloseHandle(hFile);
	*(cbuf + file_size) = '\0';

#ifdef UNICODE
	len = MultiByteToWideChar(CP_ACP, 0, cbuf, -1, NULL, 0);
	if ((buf = (TCHAR *)mem_alloc(sizeof(TCHAR) * (len + 1))) == NULL) {
		mem_free(&cbuf);
		return FALSE;
	}
	MultiByteToWideChar(CP_ACP, 0, cbuf, -1, buf, len);
	file_size = len;
	mem_free(&cbuf);
#else
	buf = cbuf;
#endif

	p = buf;
	ZeroMemory(tmp_ghsi, sizeof(GHOST_SCORE_INFO) * 2);
	winner = -1;

	// vC[擾
	for (; *p != TEXT('\0') && *p != TEXT('\r') && *p != TEXT('\n') && *p != TEXT(','); p++);
	if (*p == TEXT(',')) {
		for (; *p == TEXT(','); p++);
		if (*p == TEXT('"')) {
			for (r = p; *r != TEXT('\0') && *r != TEXT('\r') && *r != TEXT('\n') && *r != TEXT('"'); r++);
		} else {
			for (r = p; *r != TEXT('\0') && *r != TEXT('\r') && *r != TEXT('\n') && *r != TEXT(','); r++);
		}
		lstrcpyn(tmp_ghsi[0].name, p, r - p + 1);
		p = r;
		if (*p == TEXT(',')) {
			for (; *p == TEXT(','); p++);
			if (*p == TEXT('"')) {
				for (r = p; *r != TEXT('\0') && *r != TEXT('\r') && *r != TEXT('\n') && *r != TEXT('"'); r++);
			} else {
				for (r = p; *r != TEXT('\0') && *r != TEXT('\r') && *r != TEXT('\n') && *r != TEXT(','); r++);
			}
			lstrcpyn(tmp_ghsi[1].name, p, r - p + 1);
		}
	}

	// vXLbv
	for (; (file_size > (p - buf)) && *p != TEXT('\0') &&
		CompareString(MAKELCID(MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), SORT_DEFAULT),
		0, p, 4, TEXT("\r\n\r\n"), 4) != 2; p++);
	if ((file_size <= (p - buf)) || *p == TEXT('\0')) {
		mem_free(&cbuf);
		return FALSE;
	}
	p += 4;

	// XRA̓ǂݍ
	while ((file_size > (p - buf)) && *p != TEXT('\0')) {
		for (r = p; (file_size > (r - buf)) && (*r != TEXT('\r') && *r != TEXT('\n')); r++);

		if (*p == TEXT(',')) {
			// JnXRA擾
			for (; *p == TEXT(',') || *p == TEXT('*'); p++);
			tmp_ghsi[0].start_score = _ttoi(p);
			for (; *p != TEXT('\0') && *p != TEXT('\r') && *p != TEXT('\n') && *p != TEXT(','); p++);
			for (; *p == TEXT(',') || *p == TEXT('*'); p++);
			tmp_ghsi[1].start_score = _ttoi(p);

		} else if (*p >= TEXT('0') && *p <= TEXT('9')) {
			// ̓XRA擾
			round = _ttoi(p);
			if (round > 0) {
				round--;
			}
			if (round < GHOST_MAX_ROUND) {
				for (; *p != TEXT('\0') && *p != TEXT('\r') && *p != TEXT('\n') && *p != TEXT(','); p++);
				if (*p == TEXT(',')) {
					p++;
				}
				if ((*p >= TEXT('0') && *p <= TEXT('9')) || *p == TEXT('-')) {
					tmp_ghsi[0].score[round] = _ttoi(p);
					if (tmp_ghsi[0].score[round] < 0) {
						winner = 0;
					}
				} else if (*p == TEXT('(')) {
					// o[W
					p++;
					tmp_ghsi[0].score[round] = _ttoi(p) * -1;
					winner = 0;
				}
				for (; *p != TEXT('\0') && *p != TEXT('\r') && *p != TEXT('\n') && *p != TEXT(','); p++);
				if (*p == TEXT(',')) {
					p++;
				}
				for (; *p != TEXT('\0') && *p != TEXT('\r') && *p != TEXT('\n') && *p != TEXT(','); p++);
				if (*p == TEXT(',')) {
					p++;
				}
				if ((*p >= TEXT('0') && *p <= TEXT('9')) || *p == TEXT('-')) {
					tmp_ghsi[1].score[round] = _ttoi(p);
					if (tmp_ghsi[1].score[round] < 0) {
						winner = 1;
					}
				} else if (*p == TEXT('(')) {
					// o[W
					p++;
					tmp_ghsi[1].score[round] = _ttoi(p) * -1;
					winner = 1;
				}
			}

		} else if (*p == TEXT('D')) {
			// bÖړ
			if (winner >= 0) {
				alloc_ghsi = (GHOST_SCORE_INFO *)mem_calloc(sizeof(GHOST_SCORE_INFO) * (ghost->ghsi_count + 1));
				if (alloc_ghsi != NULL) {
					if (ghost->ghsi != NULL) {
						CopyMemory(alloc_ghsi, ghost->ghsi, sizeof(GHOST_SCORE_INFO) * ghost->ghsi_count);
						mem_free(&ghost->ghsi);
					}
					ghost->ghsi = alloc_ghsi;
					ghost->ghsi[ghost->ghsi_count] = tmp_ghsi[winner];
					ghost->ghsi_count++;
				}
			}
			ZeroMemory(tmp_ghsi[0].score, sizeof(TYPE_SCORE) * 100);
			ZeroMemory(tmp_ghsi[1].score, sizeof(TYPE_SCORE) * 100);
			winner = -1;
		}
		p = r;
		for (; (file_size > (p - buf)) && (*p == TEXT('\r') || *p == TEXT('\n')); p++);
	}
	mem_free(&buf);
	return TRUE;
}

/*
 *	folder_select - tH_I
 */
static BOOL folder_select(const HWND hWnd, const TCHAR *title, TCHAR *ret)
{
	BROWSEINFO bi;
	LPITEMIDLIST lpItemList;
	TCHAR buf[BUF_SIZE];

	bi.hwndOwner = hWnd;
	bi.pidlRoot = (const struct _ITEMIDLIST *)CSIDL_DESKTOP;
	*buf = TEXT('\0');
	bi.pszDisplayName = buf;
	bi.lpszTitle = title;
	bi.ulFlags = BIF_RETURNONLYFSDIRS;	/* tH_͑IłȂ悤ɂ */
	bi.lpfn = NULL;
	bi.lParam = 0;
	bi.iImage = 0;

	/* tH_Iʂ\ */
	lpItemList = SHBrowseForFolder(&bi);
	if(lpItemList == NULL){
		return FALSE;
	}
	/* IĂtH_̃tpX擾 */
	SHGetPathFromIDList(lpItemList, ret);
	return TRUE;
}

/*
 * update_start_score - JnXRA̍XV
 */
static void update_start_score(const HWND hDlg, const GHOST *ghost)
{
	TCHAR buf[BUF_SIZE];
	int i;

	SendDlgItemMessage(hDlg, IDC_COMBO_START_SOCRE, CB_RESETCONTENT, 0, 0);
	for (i = 0; i < ghost->ghsi_count; i++) {
		_itot(ghost->ghsi[i].start_score, buf, 10);
		if (SendDlgItemMessage(hDlg, IDC_COMBO_START_SOCRE, CB_FINDSTRING, -1, (LPARAM)buf) < 0) {
			SendDlgItemMessage(hDlg, IDC_COMBO_START_SOCRE, CB_ADDSTRING, 0, (LPARAM)buf);
		}
	}
	SendDlgItemMessage(hDlg, IDC_COMBO_START_SOCRE, CB_SETCURSEL, 0, 0);
}

/*
 * update_player_name - vC[̍XV
 */
static void update_player_name(const HWND hDlg, const GHOST *ghost, const int start_score)
{
	TCHAR buf[BUF_SIZE];
	int i, j, n;

	SendDlgItemMessage(hDlg, IDC_COMBO_NAME, CB_RESETCONTENT, 0, 0);
	for (i = 0; i < ghost->ghsi_count; i++) {
		if (start_score == ghost->ghsi[i].start_score) {
			j = SendDlgItemMessage(hDlg, IDC_COMBO_NAME, CB_FINDSTRING, -1, (LPARAM)ghost->ghsi[i].name);
			while (j >= 0) {
				SendDlgItemMessage(hDlg, IDC_COMBO_NAME, CB_GETLBTEXT, j, (LPARAM)buf);
				if (lstrcmp(ghost->ghsi[i].name, buf) == 0) {
					break;
				}
				n = SendDlgItemMessage(hDlg, IDC_COMBO_NAME, CB_FINDSTRING, j, (LPARAM)ghost->ghsi[i].name);
				if (j >= n) {
					j = -1;
					break;
				}
				j = n;
			}
			if (j < 0) {
				SendDlgItemMessage(hDlg, IDC_COMBO_NAME, CB_ADDSTRING, 0, (LPARAM)ghost->ghsi[i].name);
			}
		}
	}
}

/*
 * set_ghost_proc - EBhEvV[W
 */
static BOOL CALLBACK set_ghost_proc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
	GHOST *ghost;
	TCHAR buf[BUF_SIZE];
	RECT parent_rect, rect;
	int left, top;

	switch (msg) {
	case WM_INITDIALOG:
		if ((ghost = (GHOST *)lParam) == NULL) {
			EndDialog(hDlg, FALSE);
			break;
		}
		SetWindowLong(hDlg, GWL_USERDATA, lParam);

		if (*csv_path != TEXT('\0')) {
			SendDlgItemMessage(hDlg, IDC_EDIT_PATH, WM_SETTEXT, 0, (LPARAM)csv_path);
			get_csv(csv_path, ghost);
			update_start_score(hDlg, ghost);
		}

		// JnXRA
		_itot(g_start_score, buf, 10);
		SendDlgItemMessage(hDlg, IDC_COMBO_START_SOCRE, CB_SETCURSEL,
			SendDlgItemMessage(hDlg, IDC_COMBO_START_SOCRE, CB_FINDSTRING, -1, (LPARAM)buf), 0);
		update_player_name(hDlg, ghost, g_start_score);

		// vC[
		SendDlgItemMessage(hDlg, IDC_COMBO_NAME, CB_SETCURSEL,
			SendDlgItemMessage(hDlg, IDC_COMBO_NAME, CB_FINDSTRING, -1, (LPARAM)g_name), 0);

		// _
		if (g_random == 1) {
			CheckDlgButton(hDlg, IDC_CHECK_RANDOM, BST_CHECKED);
		}
		if (g_hidden == 1) {
			CheckDlgButton(hDlg, IDC_CHECK_HIDDEN, BST_CHECKED);
		}
		SendMessage(hDlg, WM_COMMAND, IDC_CHECK_RANDOM, 0);

		// U
		if (g_first == 0) {
			CheckDlgButton(hDlg, IDC_CHECK_GHOST_FIRST, BST_CHECKED);
		}

		// Z^O
		GetWindowRect(GetParent(hDlg), &parent_rect);
		GetWindowRect(hDlg, &rect);
		left = parent_rect.left + ((parent_rect.right - parent_rect.left) - (rect.right - rect.left)) / 2;
		if (left < 0) left = 0;
		top = parent_rect.top + ((parent_rect.bottom - parent_rect.top) - (rect.bottom - rect.top)) / 2;
		if (top < 0) top = 0;
		SetWindowPos(hDlg, 0, left, top, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
		break;

	case WM_CLOSE:
		EndDialog(hDlg, FALSE);
		break;

	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case IDC_BUTTON_BROWS:
			ghost = (GHOST *)GetWindowLong(hDlg, GWL_USERDATA);
			if (ghost == NULL) {
				EndDialog(hDlg, FALSE);
				break;
			}
			if (folder_select(hDlg, message_get_res(IDS_STRING_CSV_SELECT_TITLE), buf) == FALSE) {
				break;
			}
			SendDlgItemMessage(hDlg, IDC_EDIT_PATH, WM_SETTEXT, 0, (LPARAM)buf);
			get_csv(buf, ghost);
			update_start_score(hDlg, ghost);
			SendDlgItemMessage(hDlg, IDC_COMBO_START_SOCRE, WM_GETTEXT, BUF_SIZE - 1, (LPARAM)buf);
			update_player_name(hDlg, ghost, _ttoi(buf));
			break;

		case IDC_COMBO_START_SOCRE:
			ghost = (GHOST *)GetWindowLong(hDlg, GWL_USERDATA);
			if (ghost == NULL) {
				EndDialog(hDlg, FALSE);
				break;
			}
			if (HIWORD(wParam) == CBN_SELCHANGE) {
				SendDlgItemMessage(hDlg, IDC_COMBO_START_SOCRE, WM_GETTEXT, BUF_SIZE - 1, (LPARAM)buf);
				update_player_name(hDlg, ghost, _ttoi(buf));
			}
			break;

		case IDC_CHECK_RANDOM:
			if (IsDlgButtonChecked(hDlg, IDC_CHECK_RANDOM) == BST_CHECKED) {
				EnableWindow(GetDlgItem(hDlg, IDC_CHECK_HIDDEN), TRUE);
			} else {
				EnableWindow(GetDlgItem(hDlg, IDC_CHECK_HIDDEN), FALSE);
			}
			break;

		case IDOK:
			// CSV
			SendDlgItemMessage(hDlg, IDC_EDIT_PATH, WM_GETTEXT, MAX_PATH - 1, (LPARAM)csv_path);
			if (*csv_path == TEXT('\0')) {
				MessageBox(hDlg, message_get_res(IDS_STRING_ERR_CSV_SELECT), TEXT("n01_ghost"), MB_ICONEXCLAMATION);
				break;
			}

			// JnXRA
			SendDlgItemMessage(hDlg, IDC_COMBO_START_SOCRE, WM_GETTEXT, BUF_SIZE - 1, (LPARAM)buf);
			if (*buf == TEXT('\0')) {
				MessageBox(hDlg, message_get_res(IDS_STRING_ERR_START_SCORE), TEXT("n01_ghost"), MB_ICONEXCLAMATION);
				break;
			}
			g_start_score = _ttoi(buf);

			// _
			if (IsDlgButtonChecked(hDlg, IDC_CHECK_RANDOM) == BST_CHECKED) {
				g_random = 1;
			} else {
				g_random = 0;
			}
			if (IsDlgButtonChecked(hDlg, IDC_CHECK_HIDDEN) == BST_CHECKED) {
				g_hidden = 1;
			} else {
				g_hidden = 0;
			}

			// vC[
			SendDlgItemMessage(hDlg, IDC_COMBO_NAME, WM_GETTEXT, BUF_SIZE - 1, (LPARAM)g_name);
			if (*g_name == TEXT('\0') && g_random == 0) {
				MessageBox(hDlg, message_get_res(IDS_STRING_ERR_PLAYER_NAME), TEXT("n01_ghost"), MB_ICONEXCLAMATION);
				break;
			}

			// U
			if (IsDlgButtonChecked(hDlg, IDC_CHECK_GHOST_FIRST) == BST_CHECKED) {
				g_first = 0;
			} else {
				g_first = 1;
			}

			WritePrivateProfileString(TEXT("CSV"), TEXT("path"), csv_path, ini_path);
			wsprintf(buf, TEXT("%d"), g_start_score);
			WritePrivateProfileString(TEXT("OPTION"), TEXT("start_score"), buf, ini_path);
			WritePrivateProfileString(TEXT("OPTION"), TEXT("name"), g_name, ini_path);
			wsprintf(buf, TEXT("%d"), g_random);
			WritePrivateProfileString(TEXT("OPTION"), TEXT("random"), buf, ini_path);
			wsprintf(buf, TEXT("%d"), g_hidden);
			WritePrivateProfileString(TEXT("OPTION"), TEXT("hidden"), buf, ini_path);
			wsprintf(buf, TEXT("%d"), g_first);
			WritePrivateProfileString(TEXT("OPTION"), TEXT("first"), buf, ini_path);

			EndDialog(hDlg, TRUE);
			break;

		case IDCANCEL:
			SendMessage(hDlg, WM_CLOSE, 0, 0);
			break;
		}
		break;

	default:
		return FALSE;
	}
	return TRUE;
}
/* End of source */
